package kd.fi.fa.opplugin.realcard;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;
import kd.bos.algo.DataSet;
import kd.bos.algo.Row;
import kd.bos.dataentity.OperateOption;
import kd.bos.dataentity.entity.CloneUtils;
import kd.bos.dataentity.entity.DataEntityBase;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.dataentity.resource.ResManager;
import kd.bos.db.DB;
import kd.bos.db.DBRoute;
import kd.bos.db.SqlBuilder;
import kd.bos.entity.ExtendedDataEntity;
import kd.bos.entity.plugin.AddValidatorsEventArgs;
import kd.bos.entity.plugin.args.BeginOperationTransactionArgs;
import kd.bos.entity.plugin.args.EndOperationTransactionArgs;
import kd.bos.entity.validate.AbstractValidator;
import kd.bos.exception.KDBizException;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.QueryServiceHelper;
import kd.bos.servicehelper.operation.SaveServiceHelper;
import kd.fi.fa.business.cardgenerate.CardGenerateHelperEx;
import kd.fi.fa.business.operate.IFaOpHandler;
import kd.fi.fa.business.utils.AttachmentUtils;
import kd.fi.fa.business.utils.FunctionMutexHelperV2Utils;
import kd.fi.fa.common.util.Fa;
import kd.fi.fa.utils.FaOpUtils;

public class FaRealCardAuditOpEx extends FaAbstractRealCardOp implements IFaOpHandler {
    public FaRealCardAuditOpEx() {
    }

    public void onAddValidators(AddValidatorsEventArgs e) {
        super.onAddValidators(e);
        e.addValidator(new AbstractValidator() {
            public void validate() {
                ExtendedDataEntity[] dataEntities = this.getDataEntities();
                Map<Long, String> orgIdToNumber = new HashMap(4);
                ExtendedDataEntity[] var3 = dataEntities;
                int var4 = dataEntities.length;

                for(int var5 = 0; var5 < var4; ++var5) {
                    ExtendedDataEntity dataEntity = var3[var5];
                    long orgId = dataEntity.getDataEntity().getLong(Fa.id("org"));
                    if (orgIdToNumber.get(orgId) == null) {
                        String number = dataEntity.getDataEntity().getDynamicObject("org").getString("number");
                        orgIdToNumber.put(orgId, number);
                    }
                }

                DynamicObjectCollection mainBooks = QueryServiceHelper.query("fa_assetbook", "org", new QFilter[]{new QFilter("org", "in", orgIdToNumber.keySet()), new QFilter("ismainbook", "=", true)});
                Set<Long> bookOrgIds = (Set)mainBooks.stream().map((v) -> {
                    return v.getLong("org");
                }).collect(Collectors.toSet());
                String numbers = "";
                Iterator var13 = orgIdToNumber.entrySet().iterator();

                while(var13.hasNext()) {
                    Map.Entry<Long, String> entry = (Map.Entry)var13.next();
                    if (!bookOrgIds.contains(entry.getKey())) {
                        numbers = "、" + (String)entry.getValue();
                    }
                }

                if (!numbers.isEmpty()) {
                    throw new KDBizException(String.format(ResManager.loadKDString("核算组织[%s]不存在主账簿。", "FaRealCardAuditOp_0", "fi-fa-opplugin", new Object[0]), numbers.substring(1)));
                }
            }
        });
    }

    public void beginOperationTransaction(BeginOperationTransactionArgs e) {
        String operationKey = e.getOperationKey();
        Map<String, Object> otherParams = new HashMap(1, 1.0F);
        otherParams.put("beginArgs", e);
        FunctionMutexHelperV2Utils.doFaOpWithWithOrgPcChildLock(operationKey, "fa_card_real", e.getDataEntities(), otherParams, this);
    }

    public void endOperationTransaction(EndOperationTransactionArgs e) {
        super.endOperationTransaction(e);
        this.updateAuditInfo4OriReal(e);
    }

    public void doFaOp(String operationKey, String billName, DynamicObject[] dataEntities, Map<String, Object> otherParams) {
        BeginOperationTransactionArgs e = (BeginOperationTransactionArgs)otherParams.get("beginArgs");
        OperateOption option = this.getOption();
        boolean assetCardOperate = option.containsVariable("assetCardOperate");
        this.createOriginalReal(e);
        if (!assetCardOperate) {
            this.createFinCard(e);
        }

        super.restoreInitFields(e);
    }

    private void createOriginalReal(BeginOperationTransactionArgs e) {
        DynamicObject[] dataEntities = e.getDataEntities();
        List<Long> realCardIds = new ArrayList(dataEntities.length);
        DynamicObject[] var4 = dataEntities;
        int var5 = dataEntities.length;

        for(int var6 = 0; var6 < var5; ++var6) {
            DynamicObject data = var4[var6];
            realCardIds.add(data.getLong("id"));
        }

        Collection<DynamicObject> realCards = BusinessDataServiceHelper.loadFromCache("fa_card_real", new QFilter[]{new QFilter("id", "in", realCardIds)}).values();
        List<DynamicObject> oriRealCards = new ArrayList(realCards.size());
        long[] ids = DB.genLongIds("t_fa_card_real", realCards.size());
        Map<Object, List<Object>> sourceId2TargetIdMap = new HashMap(realCards.size());
        int i = 0;

        for(Iterator var9 = realCards.iterator(); var9.hasNext(); ++i) {
            DynamicObject realCard = (DynamicObject)var9.next();
            DynamicObject oriRealCard = (DynamicObject)(new CloneUtils(true, true)).clone(realCard);
            oriRealCard.set("id", ids[i]);
            oriRealCard.set("isbak", Boolean.TRUE);
            oriRealCard.set("originaldata", Boolean.TRUE);
            oriRealCards.add(oriRealCard);
            List<Object> targetIds = new ArrayList(1);
            targetIds.add(ids[i]);
            sourceId2TargetIdMap.put(realCard.getLong("id"), targetIds);
        }

        SaveServiceHelper.save((DynamicObject[])oriRealCards.toArray(new DynamicObject[0]));
        Set<Object> pks = (Set)realCards.stream().map(DataEntityBase::getPkValue).collect(Collectors.toSet());
        SqlBuilder sql = new SqlBuilder();
        sql.append("update t_fa_card_real set foriginaldata = '0' where foriginaldata = '1' and ", new Object[0]);
        sql.appendIn(" fid ", pks.toArray(new Object[0]));
        DB.execute(DBRoute.of("fa"), sql);
        AttachmentUtils.copyAttachments("fa_card_real", "fa_card_real", "attachmentpanel", sourceId2TargetIdMap);
    }

    private void updateAuditInfo4OriReal(EndOperationTransactionArgs e) {
        DynamicObject[] dataEntities = e.getDataEntities();
        List<Long> realCardIds = new ArrayList(dataEntities.length);
        DynamicObject[] var4 = dataEntities;
        int var5 = dataEntities.length;

        for(int var6 = 0; var6 < var5; ++var6) {
            DynamicObject data = var4[var6];
            realCardIds.add(data.getLong("id"));
        }

        List<Object[]> params = new ArrayList(realCardIds.size());
        SqlBuilder querySql = new SqlBuilder();
        querySql.append("select fmasterid,fbillstatus,fbizstatus,fauditdate,fauditorid from t_fa_card_real where ", new Object[0]);
        querySql.appendIn(" fid ", realCardIds.toArray(new Object[0]));
        DataSet dataSet = DB.queryDataSet(this.getClass().getName(), DBRoute.of("fa"), querySql);
        Throwable var24 = null;

        try {
            Iterator var8 = dataSet.iterator();

            while(var8.hasNext()) {
                Row row = (Row)var8.next();
                Object[] auditInfo = new Object[]{row.getString("fbillstatus"), row.getString("fbizstatus"), row.getDate("fauditdate"), row.getLong("fauditorid"), row.getLong("fmasterid")};
                params.add(auditInfo);
            }
        } catch (Throwable var18) {
            var24 = var18;
            throw var18;
        } finally {
            if (dataSet != null) {
                if (var24 != null) {
                    try {
                        dataSet.close();
                    } catch (Throwable var17) {
                        var24.addSuppressed(var17);
                    }
                } else {
                    dataSet.close();
                }
            }

        }

        String updateSql = "update t_fa_card_real set fbillstatus = ?,fbizstatus = ?,fauditdate = ?,fauditorid = ? where fmasterid = ? and foriginaldata = '1';";
        DB.executeBatch(DBRoute.of("fa"), updateSql, params);
    }

    private void createFinCard(BeginOperationTransactionArgs e) {
        Map<Object, List<DynamicObject>> orgDynamicObjectMap = this.orgDynamicObjectMap(e);
        Iterator var3 = orgDynamicObjectMap.entrySet().iterator();

        while(var3.hasNext()) {
            Map.Entry<Object, List<DynamicObject>> entry = (Map.Entry)var3.next();
            CardGenerateHelperEx.generateFinCard((List)entry.getValue());
        }

    }

    private Map<Object, List<DynamicObject>> orgDynamicObjectMap(BeginOperationTransactionArgs e) {
        Map<Object, List<DynamicObject>> orgCardMap = new HashMap(16);
        Iterator var3 = FaOpUtils.getDataEntities(e).iterator();

        while(var3.hasNext()) {
            DynamicObject realCard = (DynamicObject)var3.next();
            Object orgPk = realCard.getDynamicObject("org").getPkValue();
            List<DynamicObject> cards = (List)orgCardMap.computeIfAbsent(orgPk, (v) -> {
                return new ArrayList();
            });
            cards.add(realCard);
        }

        return orgCardMap;
    }
}
